#region Using declarations
using System;
using System.Linq;
using System.Windows;
using System.Windows.Media;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.Tools;
using System.Windows.Controls;
#endregion

/******************************************************************************
**************************** NinjaAddons.com **********************************
*******************************************************************************
* Authors NT7 : Credit to NT Forum and NT for the original NT7 indicator
* Author NT8 : Kriss { Ninja-Addons.com / AzurITec.com}
*******************************************************************************
~~~~~~~~~~~~~~~~~~~~~~~~~~~~ info version ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
###############################################################################
v1.00  - 20.04.2016    + Conversion from NT7 to NT8b10. 
                        + First version as indicator
-------------------------------------------------------------------------------
v1.01  - 10.11.2017     + Finally had time to create an add-on instead
                        + Removed Snap Mode Buttons
                        + Set Toolbar's Margins / Padding / HorizontalAlignment
                        + works on NT8.0.10.0
-------------------------------------------------------------------------------
v1.02  - 17.09.2017     + Added new "Time Cycles" and "Polygon" => Credits to PrTester : https://ninjatrader.com/support/forum/member.php?u=1235
                        + Added new "Order Flow Volume Profile"
                        + works on NT8.0.15.1
-------------------------------------------------------------------------------
###############################################################################
-------------------------------------------------------------------------------
=>  This code is open source and given for information purpose only.
=>  Please let this section -- Author / Info -- as is
=>  And Feel free to share !!
=>  Send me an email at contact@ninja-addons.com if you've found some bugs or 
    have added some features.
-------------------------------------------------------------------------------
******************** LIMITATION OF LIABILITY **********************************
-------------------------------------------------------------------------------
IN NO EVENT SHALL AZUR INVESTMENT TECHNOLOGIES SAS BE LIABLE FOR ANY 
DAMAGES (INCLUDING, WITHOUT LIMITATION, LOST PROFITS, BUSINESS INTERRUPTION, 
OR LOST INFORMATION) RISING OUT OF �AUTHORIZED USERS� USE OF OR INABILITY 
TO USE THIS SOFTWARE, EVEN IF AZUR INVESTMENT TECHNOLOGIES SAS HAS 
BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. IN NO EVENT WILL AZUR 
INVESTMENT TECHNOLOGIES SAS BE LIABLE FOR LOSS OF DATA OR FOR INDIRECT, 
SPECIAL, INCIDENTAL, CONSEQUENTIAL (INCLUDING LOST PROFIT), OR OTHER 
DAMAGES BASED IN CONTRACT, TORT OR OTHERWISE. AZUR INVESTMENT TECHNOLOGIES 
SAS SHALL HAVE NO LIABILITY WITH RESPECT TO THE CONTENT OF THIS SOFTWARE 
OR ANY PART THEREOF, INCLUDING BUT NOT LIMITED TO ERRORS OR OMISSIONS 
CONTAINED THEREIN, LIBEL, INFRINGEMENTS OF RIGHTS OF PUBLICITY, PRIVACY, 
TRADEMARK RIGHTS, BUSINESS INTERRUPTION, PERSONAL INJURY, LOSS OF PRIVACY, 
MORAL RIGHTS OR THE DISCLOSURE OF CONFIDENTIAL INFORMATION.
-------------------------------------------------------------------------------
###############################################################################
-----------------------------------------------------------------------------*/


#region -- the code --
namespace NinjaTrader.NinjaScript.AddOns.NinjaAddons
{
    public class NinjaAddonsToolBar : AddOnBase
    {
        //---------------- Fields / Const -------------------
        #region -- Constantes Values based on NT UI (can be modified in other NT8 release) --
        private const int DRAWINGTOOLS_INDEX = 3;
        private const double ITEM_WIDTH = 20;
        private const string STAYINDRAWMODE_HEADER = "Stay In Draw Mode";
        private const string DRAWINGOBJECTS_HEADER = "Drawing Objects...";
        private const string SNAPMODE_HEADER = "Snap Mode";
        private string[] MISC_HEADERS = new string[4] { "Ruler", "Risk-Reward", "Region Highlight X", "Region Highlight Y" };
        private string[] LINE_HEADERS = new string[6] { "Line", "Ray", "Extended Line", "Arrow Line", "Horizontal Line", "Vertical Line" };
        private string[] FIBS_HEADERS = new string[4] { "Fibonacci Retracements", "Fibonacci Extensions", "Fibonacci Time Extensions", "Fibonacci Circle" };
        private string[] CHANNELS_HEADERS = new string[5] { "Andrew's Pitchfork", "Gann Fan", "Regression Channel", "Trend Channel", "Time Cycles" };
        private string[] SHAPES_HEADERS = new string[7] { "Ellipse", "Rectangle", "Triangle", "Polygon", "Order Flow Volume Profile", "Arc", "Text" };
        private const string CHARTMARKER_HEADER = "Chart Marker";
        private const string ARROWUP_HEADER = "Arrow Up";
        private const string ARROWDOWN_HEADER = "Arrow Down";
        private const string TRIANGLEUP_HEADER = "Triangle Up";
        private const string TRIANGLEDOWN_HEADER = "Triangle Down";
        private const string REMOVEOBJECTS_HEADER = "Remove All Drawing Objects";
        #endregion

		private const string TOOLBARSHORTCUT_NAME = "ShortCutToolbar";

        //---------------- virtuals -------------------
        #region -- protected override void OnStateChange() --
        protected override void OnStateChange()
        {
            if (State == State.SetDefaults) Name = "Toolbar Shortcut by NinjaAddons.com";
        }
        #endregion

        #region -- protected override void OnWindowCreated(Window window) --
        protected override void OnWindowCreated(Window window)
        {
            if (window != null)
            {
                window.Dispatcher.InvokeAsync((Action)(() =>
                {
                    var chart = window as Gui.Chart.Chart;
                    if (chart == null) return;

                    var chartGrid = (chart.FindFirst("ChartWindowChartTraderControl") as ChartTrader).Parent as Grid;
                    if (chartGrid == null) return;

                    //if toolbar already exists, do not recreate it
                    foreach (var v in chartGrid.Children)
                        if (v is VirtualizingStackPanel && (v as VirtualizingStackPanel).Uid == TOOLBARSHORTCUT_NAME) return;

                    #region -- Create Toolbar Container --
                    VirtualizingStackPanel verticalStackPanel = new VirtualizingStackPanel()
                    {
                        Name = TOOLBARSHORTCUT_NAME,
                        Uid = TOOLBARSHORTCUT_NAME,
                        Orientation = System.Windows.Controls.Orientation.Vertical,
                        VerticalAlignment = VerticalAlignment.Stretch,
                        HorizontalAlignment = HorizontalAlignment.Stretch,
                        Margin = new Thickness(0, 0, 7, 0)//Margin is used for separators
                    };
                    #endregion

                    #region -- Get all Drawing Items and Copy there Style and Events to new Drawing Items --
                    NTMenuItem ntDrawingTools = ((Menu)chart.MainMenu[DRAWINGTOOLS_INDEX]).Items[0] as NTMenuItem;
                    foreach (var item in ntDrawingTools.Items)
                    {
						if (item is NTMenuItem && ((NTMenuItem)item).Header != null)
                        {
                            NTMenuItem mnItem = item as NTMenuItem;
							Print(mnItem.Header.ToString());
                        
                            if (mnItem.Header.ToString() == STAYINDRAWMODE_HEADER || mnItem.Header.ToString() == DRAWINGOBJECTS_HEADER || mnItem.Header.ToString() == SNAPMODE_HEADER) continue;

                            #region -- Add Misc --
                            else if (MISC_HEADERS.Contains(mnItem.Header.ToString()))
                            {
                                verticalStackPanel.Children.Add(new NTMenuItem()
                                {
                                    ToolTip = mnItem.Header,
                                    Style = mnItem.Style,
                                    Icon = mnItem.Icon,
                                    Command = mnItem.Command,
                                    Margin = new Thickness(1),
                                    Width = ITEM_WIDTH,
                                    HorizontalAlignment =
                                    HorizontalAlignment.Left
                                });
                                if (mnItem.Header.ToString() == MISC_HEADERS.Last()) verticalStackPanel.Children.Add(new Separator());
                            }
                            #endregion

                            #region -- Add Lines --
                            else if (LINE_HEADERS.Contains(mnItem.Header.ToString()))
                            {
                                verticalStackPanel.Children.Add(new NTMenuItem()
                                {
                                    ToolTip = mnItem.Header,
                                    Style = mnItem.Style,
                                    Icon = mnItem.Icon,
                                    Command = mnItem.Command,
                                    Margin = new Thickness(1),
                                    Width = ITEM_WIDTH,
                                    HorizontalAlignment =
                                    HorizontalAlignment.Left
                                });
                                if (mnItem.Header.ToString() == LINE_HEADERS.Last()) verticalStackPanel.Children.Add(new Separator());
                            }
                            #endregion

                            #region -- Add Fibs --
                            else if (FIBS_HEADERS.Contains(mnItem.Header.ToString()))
                            {
                                verticalStackPanel.Children.Add(new NTMenuItem()
                                {
                                    ToolTip = mnItem.Header,
                                    Style = mnItem.Style,
                                    Icon = mnItem.Icon,
                                    Command = mnItem.Command,
                                    Margin = new Thickness(1),
                                    Width = ITEM_WIDTH,
                                    HorizontalAlignment =
                                    HorizontalAlignment.Left
                                });
                                if (mnItem.Header.ToString() == FIBS_HEADERS.Last()) verticalStackPanel.Children.Add(new Separator());
                            }
                            #endregion

                            #region -- Add Channels --
                            else if (CHANNELS_HEADERS.Contains(mnItem.Header.ToString()))
                            {
                                verticalStackPanel.Children.Add(new NTMenuItem()
                                {
                                    ToolTip = mnItem.Header,
                                    Style = mnItem.Style,
                                    Icon = mnItem.Icon,
                                    Command = mnItem.Command,
                                    Margin = new Thickness(1),
                                    Width = ITEM_WIDTH,
                                    HorizontalAlignment =
                                    HorizontalAlignment.Left
                                });
                                if (mnItem.Header.ToString() == CHANNELS_HEADERS.Last()) verticalStackPanel.Children.Add(new Separator());
                            }
                            #endregion

                            #region -- Add Shapes --
                            else if (SHAPES_HEADERS.Contains(mnItem.Header.ToString()))
                            {
                                verticalStackPanel.Children.Add(new NTMenuItem()
                                {
                                    ToolTip = mnItem.Header,
                                    Style = mnItem.Style,
                                    Icon = mnItem.Icon,
                                    Command = mnItem.Command,
                                    Margin = new Thickness(1),
                                    Width = ITEM_WIDTH,
                                    HorizontalAlignment =
                                    HorizontalAlignment.Left
                                });
                                if (mnItem.Header.ToString() == SHAPES_HEADERS.Last()) verticalStackPanel.Children.Add(new Separator());
                            }
                            #endregion

                            #region -- Add Markers --
                            else if (mnItem.Header.ToString() == CHARTMARKER_HEADER)
                            {
                                foreach (var subitem in mnItem.Items)
                                {
                                    if (subitem is NTMenuItem)
                                    {
                                        NTMenuItem subitemMnItem = subitem as NTMenuItem;
                                        NTMenuItem newSubItem = new NTMenuItem()
                                        {
                                            ToolTip = subitemMnItem.Header,
                                            Style = subitemMnItem.Style,
                                            Icon = subitemMnItem.Icon,
                                            Command = subitemMnItem.Command,
                                            Margin = new Thickness(1),
                                            Width = ITEM_WIDTH,
                                            HorizontalAlignment = HorizontalAlignment.Left
                                        };

                                        if (subitemMnItem.Header.ToString() == ARROWUP_HEADER || subitemMnItem.Header.ToString() == TRIANGLEUP_HEADER)
                                            newSubItem.Foreground = Brushes.Green;
                                        else if (subitemMnItem.Header.ToString() == ARROWDOWN_HEADER || subitemMnItem.Header.ToString() == TRIANGLEDOWN_HEADER)
                                            newSubItem.Foreground = Brushes.Red;

                                        verticalStackPanel.Children.Add(newSubItem);
                                    }
                                }
                                verticalStackPanel.Children.Add(new Separator());
                            }
                            #endregion

                            #region -- Add Remove Objects --
                            else if (mnItem.Header.ToString() == REMOVEOBJECTS_HEADER)
                            {
                                verticalStackPanel.Children.Add(new NTMenuItem()
                                {
                                    ToolTip = mnItem.Header,
                                    Style = mnItem.Style,
                                    Icon = mnItem.Icon,
                                    Command = mnItem.Command,
                                    Margin = new Thickness(1),
                                    Width = ITEM_WIDTH,
                                    HorizontalAlignment =
                                    HorizontalAlignment.Left
                                });
                            }
                            #endregion
                        }
                    }
                    #endregion

                    #region -- Add Toolbar in first column of chart grid --
                    chartGrid.ColumnDefinitions.Insert(0, new ColumnDefinition());
                    chartGrid.ColumnDefinitions[0].Width = new GridLength(ITEM_WIDTH + 12);
                    chartGrid.Margin = new Thickness(0);

                    // loop over existing items and move those to the newly created columns and rows
                    for (int i = 0; i < chartGrid.Children.Count; i++)
                        Grid.SetColumn(chartGrid.Children[i], Grid.GetColumn(chartGrid.Children[i]) + 1);

                    chartGrid.Children.Add(verticalStackPanel);
                    Grid.SetColumn(verticalStackPanel, 0);
                    #endregion
                }));
            }
        }

        private void Tmp_Click(object sender, RoutedEventArgs e)
        {
            throw new NotImplementedException();
        }
        #endregion

        #region -- protected override void OnWindowDestroyed(Window window) --
        protected override void OnWindowDestroyed(Window window)
        {
            var chart = window as Chart;
            if (chart == null) return;

            var chartGrid = (chart.FindFirst("ChartWindowChartTraderControl") as ChartTrader).Parent as Grid;
            if (chartGrid == null) return;

            VirtualizingStackPanel verticalStackPanel = null;
            foreach (var v in chartGrid.Children)
                if (v is VirtualizingStackPanel && (v as VirtualizingStackPanel).Uid == "ShortCutToolbar") { verticalStackPanel = v as VirtualizingStackPanel; break; }
            if (verticalStackPanel != null)
                chartGrid.Children.Remove(verticalStackPanel);

            for (int i = 0; i < chartGrid.ColumnDefinitions.Count - 1; i++)
                chartGrid.ColumnDefinitions[i].Width = chartGrid.ColumnDefinitions[i + 1].Width;

            for (int i = 0; i < chartGrid.Children.Count; i++)
                if ((Grid.GetColumn(chartGrid.Children[i]) - 1) >= 0)
                    Grid.SetColumn(chartGrid.Children[i], Grid.GetColumn(chartGrid.Children[i]) - 1);

            chartGrid.ColumnDefinitions.RemoveRange(chartGrid.ColumnDefinitions.Count - 1, 1);
        }
        #endregion
    }
}
#endregion