// 
// Copyright (C) 2016, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//
#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

// This namespace holds indicators in this folder and is required. Do not change it.
namespace NinjaTrader.NinjaScript.Indicators
{
	/// <summary>
	/// Volume is simply the number of shares (or contracts) traded during a specified time frame (e.g. hour, day, week, month, etc).
	/// </summary>
	public class VOL : Indicator
	{
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description					= NinjaTrader.Custom.Resource.NinjaScriptIndicatorDescriptionVOL;
				Name						= NinjaTrader.Custom.Resource.NinjaScriptIndicatorNameVOL;
				BarsRequiredToPlot			= 0;
				Calculate					= Calculate.OnEachTick;
				IsSuspendedWhileInactive	= true;

				AddPlot(new Stroke(Brushes.Blue, 2), PlotStyle.Bar,	NinjaTrader.Custom.Resource.VOLVolume);
				AddLine(Brushes.DarkGray, 0,						NinjaTrader.Custom.Resource.NinjaScriptIndicatorZeroLine);
			}
		}

		protected override void OnBarUpdate()
		{
			Value[0] = Volume[0];
		}
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private VOL[] cacheVOL;
		public VOL VOL()
		{
			return VOL(Input);
		}

		public VOL VOL(ISeries<double> input)
		{
			if (cacheVOL != null)
				for (int idx = 0; idx < cacheVOL.Length; idx++)
					if (cacheVOL[idx] != null &&  cacheVOL[idx].EqualsInput(input))
						return cacheVOL[idx];
			return CacheIndicator<VOL>(new VOL(), input, ref cacheVOL);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.VOL VOL()
		{
			return indicator.VOL(Input);
		}

		public Indicators.VOL VOL(ISeries<double> input )
		{
			return indicator.VOL(input);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.VOL VOL()
		{
			return indicator.VOL(Input);
		}

		public Indicators.VOL VOL(ISeries<double> input )
		{
			return indicator.VOL(input);
		}
	}
}

#endregion
