#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
// This indicator was developed by Shanta Santaprakash 
// Purpose of this indicator is to color chart regions based on ADX Value, ADX Slope and EMA Slope
//Green: Rising ADX & Rising EMA
//Blue: Falling ADX & Rising EMA
//Red: Rising ADX & Falling EMA
//Yellow Falling ADX and Falling EMA
// Minimum ADX value is customizable. Default is 15
//When ADX is below minimum ADX, no color 
//THis is merely a technical indicator and is not a trade suggestion. 
// Copyright Shanta Santaprakash

namespace NinjaTrader.NinjaScript.Indicators.Shanta
{
	public class ADXRegions : Indicator
	{
		private ADX ADX1;
		private EMA EMA1;
		private Momentum Momentum1;
		private string adxcolor = @"white";
		
		 
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Enter the description for your new custom Indicator here.";
				Name										= "ADXRegions";
				Calculate									= Calculate.OnBarClose;
				IsOverlay									= false;
				DisplayInDataBox							= true;
				DrawOnPricePanel							= true;
				DrawHorizontalGridLines						= true;
				DrawVerticalGridLines						= true;
				PaintPriceMarkers							= true;
				ScaleJustification							= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				IsSuspendedWhileInactive					= true;
				ADX2 										= 15;
				
				
				
			}
			else if (State == State.Configure)
			{
			}
			else if (State == State.DataLoaded)
			{				
				ADX1				= ADX(Close, 14);
				EMA1				= EMA(Close, 14);
				Momentum1				= Momentum(EMA1, 2);
			}		
		}

		protected override void OnBarUpdate()
		{
			//Add your custom indicator logic here.
			if (BarsInProgress != 0) 
				return;

			if (CurrentBars[0] < 1)
			return;

			if (ADX1[0] > ADX2)
			 // Set 1

			{
				if ((ADX1[0] > ADX1[1])
					&& (Momentum1[0] < 0))	
				{
					Draw.RegionHighlightX(this, @"ADXRegion " + CurrentBars[0].ToString(), 1, 0, Brushes.Transparent, Brushes.Red, 20);
				}
			
			 // Set 2
				 if ((ADX1[0] > ADX1[1])
					 && (Momentum1[0] > 0))
				{
				Draw.RegionHighlightX(this, @"ADXRegion " + CurrentBars[0].ToString(), 1, 0, Brushes.Transparent, Brushes.Green, 20);
				}
			 // Set 3

				if ((ADX1[0] < ADX1[1])
					&& (Momentum1[0] < 0))	
				{
					Draw.RegionHighlightX(this, @"ADXRegion " + CurrentBars[0].ToString(), 1, 0, Brushes.Transparent, Brushes.Yellow, 20);
				}
			
			 // Set 4
				 if ((ADX1[0] < ADX1[1])
					 && (Momentum1[0] > 0))
				{
				Draw.RegionHighlightX(this, @"ADXRegion " + CurrentBars[0].ToString(), 1, 0, Brushes.Transparent, Brushes.Blue, 20);
				adxcolor = @"Red";
				}
				
			}
			else
			
				{
					adxcolor = @"white";
				}
		}			
		
		#region Properties
		
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="ADX", Order=1, GroupName="Parameters")]
		public int ADX2
		{ get; set; }
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private Shanta.ADXRegions[] cacheADXRegions;
		public Shanta.ADXRegions ADXRegions(int aDX2)
		{
			return ADXRegions(Input, aDX2);
		}

		public Shanta.ADXRegions ADXRegions(ISeries<double> input, int aDX2)
		{
			if (cacheADXRegions != null)
				for (int idx = 0; idx < cacheADXRegions.Length; idx++)
					if (cacheADXRegions[idx] != null && cacheADXRegions[idx].ADX2 == aDX2 && cacheADXRegions[idx].EqualsInput(input))
						return cacheADXRegions[idx];
			return CacheIndicator<Shanta.ADXRegions>(new Shanta.ADXRegions(){ ADX2 = aDX2 }, input, ref cacheADXRegions);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.Shanta.ADXRegions ADXRegions(int aDX2)
		{
			return indicator.ADXRegions(Input, aDX2);
		}

		public Indicators.Shanta.ADXRegions ADXRegions(ISeries<double> input , int aDX2)
		{
			return indicator.ADXRegions(input, aDX2);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.Shanta.ADXRegions ADXRegions(int aDX2)
		{
			return indicator.ADXRegions(Input, aDX2);
		}

		public Indicators.Shanta.ADXRegions ADXRegions(ISeries<double> input , int aDX2)
		{
			return indicator.ADXRegions(input, aDX2);
		}
	}
}

#endregion
