#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Indicators in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Indicators
{
	public class VolumeGT : Indicator
	{
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description					= @"Volume with Moving Average for Golden Triangle, TASC September 2014";
				Name						= "VolumeGT";
				Calculate					= Calculate.OnBarClose;
				IsOverlay					= false;
				DisplayInDataBox			= true;
				DrawOnPricePanel			= true;
				DrawHorizontalGridLines		= true;
				DrawVerticalGridLines		= true;
				PaintPriceMarkers			= true;
				ScaleJustification			= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				IsSuspendedWhileInactive	= true;
				SmaPeriod					= 1;

				AddPlot(new Stroke(Brushes.DodgerBlue, 2), PlotStyle.Bar, "Volume");
				AddPlot(Brushes.Orange, "MovingAverage");
			}
		}

		protected override void OnBarUpdate()
		{
			Volume[0] = VOL()[0];
            MovingAverage[0] = SMA(Volume, SmaPeriod)[0];
		}

		#region Properties
		[Range(1, int.MaxValue)]
		[NinjaScriptProperty]
		[Display(Name="SmaPeriod", Description="Numbers of bars used for calculations", Order=1, GroupName="Parameters")]
		public int SmaPeriod
		{ get; set; }

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> Volume
		{
			get { return Values[0]; }
		}

		[Browsable(false)]
		[XmlIgnore]
		public Series<double> MovingAverage
		{
			get { return Values[1]; }
		}
		#endregion

	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private VolumeGT[] cacheVolumeGT;
		public VolumeGT VolumeGT(int smaPeriod)
		{
			return VolumeGT(Input, smaPeriod);
		}

		public VolumeGT VolumeGT(ISeries<double> input, int smaPeriod)
		{
			if (cacheVolumeGT != null)
				for (int idx = 0; idx < cacheVolumeGT.Length; idx++)
					if (cacheVolumeGT[idx] != null && cacheVolumeGT[idx].SmaPeriod == smaPeriod && cacheVolumeGT[idx].EqualsInput(input))
						return cacheVolumeGT[idx];
			return CacheIndicator<VolumeGT>(new VolumeGT(){ SmaPeriod = smaPeriod }, input, ref cacheVolumeGT);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.VolumeGT VolumeGT(int smaPeriod)
		{
			return indicator.VolumeGT(Input, smaPeriod);
		}

		public Indicators.VolumeGT VolumeGT(ISeries<double> input , int smaPeriod)
		{
			return indicator.VolumeGT(input, smaPeriod);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.VolumeGT VolumeGT(int smaPeriod)
		{
			return indicator.VolumeGT(Input, smaPeriod);
		}

		public Indicators.VolumeGT VolumeGT(ISeries<double> input , int smaPeriod)
		{
			return indicator.VolumeGT(input, smaPeriod);
		}
	}
}

#endregion
