#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion
///
/// Written by @ aligator on Ninjatrader and Futures.IO Forums - December 13, 2018
/// This is an Implementation of Ken Calhoun's Dual-Candle Momentum Breakouts, December 2018 TASC
/// 
/// The indicator uses an 8 period HMA for trend direction intended for intraday trades
/// The HMA period can be increased for trading stocks on daily charts. The indicator also exposes signals
/// for Market Analizer which is very useful tool for scaning a stock list Overlay the indicator in panel 1.
/// 
/// Notes: Best trades are signals near the top/bottom of a swing following a reversal pattern such as a Doji,
/// 	   Hangman, Hammer, Engulfing, etc. bar. Other signals can be considered as continuation signals
/// 	   following retracements to initiate a new position or add to a current position. Confirm signal
/// 	   with your favorite momentum indicator.
/// 
/// 	   For daily stocks, Calhoun recommends using $20 - $70 stocks and enter the trade 50 cents above the
/// 	   high of the signal bar with a $2 stop in an uptrend.
/// 
/// Exported: NT 8.0.16.3
/// 
namespace NinjaTrader.NinjaScript.Indicators.mah_Indicators
{
	public class mahDualCandlePattern : Indicator
	{
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description					= @"Implementation of Ken Calhoun's Dual-Candle Momentum Breakouts, December 2018 TASC - Created by : aligator"+
											  "Notes: Best trades are signals near the top/bottom of a swing following a reversal pattern such as a Doji, Hangman,"+
											  "Hammer, Engulfing, etc. bar. Other signals can be considered as continuation signals following retracements"+
											  "to initiate a new position or add to a current position. Confirm signal with your favorite momentum indicator."+

											  "For daily stocks, Calhoun recommends using $20 - $70 stocks and enter the trade 50 cents above the high of"+
											  "the signal bar with a $2 stop in an uptrend.";

				Name						= "mahDualCandlePattern";
				Calculate					= Calculate.OnBarClose;
				IsOverlay					= true;
				DisplayInDataBox			= true;
				DrawOnPricePanel			= true;
				DrawHorizontalGridLines		= true;
				DrawVerticalGridLines		= true;
				PaintPriceMarkers			= true;
				ScaleJustification			= NinjaTrader.Gui.Chart.ScaleJustification.Right;
				//Disable this property if your indicator requires custom values that cumulate with each new market data event. 
				//See Help Guide for additional information.
				
				AddPlot(Brushes.Transparent, "MAAnalyzer");				
				IsSuspendedWhileInactive	= true;
				Period						= 8;
				Multiplier					= 1.618; // Change to 1 for Calhoun's method. Use larger than 1 for stronger and less frequent signals
			}
			else if (State == State.Configure)
			{
			}
		}

		protected override void OnBarUpdate()
		{
			if (CurrentBar < 1 )
				return;
			
			// Bullish Dual-Candle
			if (IsRising(HMA(Period)) && Close[1] > Open[1] && Close[0] > Open[0] && Close[0] > Close[1] && (Close[0]-Open[0]) >= Multiplier * (Close[1]-Open[1]))
			{
				if (ChartBars != null)
				{
					BarBrushes[1]			= Brushes.Lime;
					BarBrush				= Brushes.Lime;
					CandleOutlineBrushes[0] = Brushes.Blue;
				}

					Draw.Text(this, "Bullish DualCandle" + CurrentBar, false, "DualC" , 0, Math.Min(Low[0], Low[1]), -10, Brushes.GreenYellow,
						new SimpleFont("Arial", 10), TextAlignment.Center, Brushes.Transparent, Brushes.Transparent, 0);
							
					MAAnalyzer[0] = 1;
			}
			
			// Bearish Dual-Candl			
			if (IsFalling(HMA(Period)) && Close[1] < Open[1] && Close[0] < Open[0] && Close[0] < Close[1] && (Open[0]-Close[0]) >= Multiplier * (Open[1]-Close[1]))
			{
				if (ChartBars != null)
				{
					BarBrushes[1]			= Brushes.Red;
					BarBrush				= Brushes.Red;
					CandleOutlineBrushes[0] = Brushes.Yellow;
				}

					Draw.Text(this, "Bearish DualCandle" + CurrentBar, false, "DualC" , 0, Math.Max(High[0], High[1]), 10, Brushes.Orange,
						new SimpleFont("Arial", 10), TextAlignment.Center, Brushes.Transparent, Brushes.Transparent, 0);
							
					MAAnalyzer[0] = 2;							
			}						
		}
		
		#region Properties
		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> MAAnalyzer
		{
			get { return Values[0]; }
		}		
		
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="Period", Order=1, GroupName="Parameters")]
		public int Period
		{ get; set; }
		
		[NinjaScriptProperty]
		[Display(Name="Multiplier", Order=2, GroupName="Parameters")]
		public double Multiplier
		{ get; set; }		
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private mah_Indicators.mahDualCandlePattern[] cachemahDualCandlePattern;
		public mah_Indicators.mahDualCandlePattern mahDualCandlePattern(int period, double multiplier)
		{
			return mahDualCandlePattern(Input, period, multiplier);
		}

		public mah_Indicators.mahDualCandlePattern mahDualCandlePattern(ISeries<double> input, int period, double multiplier)
		{
			if (cachemahDualCandlePattern != null)
				for (int idx = 0; idx < cachemahDualCandlePattern.Length; idx++)
					if (cachemahDualCandlePattern[idx] != null && cachemahDualCandlePattern[idx].Period == period && cachemahDualCandlePattern[idx].Multiplier == multiplier && cachemahDualCandlePattern[idx].EqualsInput(input))
						return cachemahDualCandlePattern[idx];
			return CacheIndicator<mah_Indicators.mahDualCandlePattern>(new mah_Indicators.mahDualCandlePattern(){ Period = period, Multiplier = multiplier }, input, ref cachemahDualCandlePattern);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.mah_Indicators.mahDualCandlePattern mahDualCandlePattern(int period, double multiplier)
		{
			return indicator.mahDualCandlePattern(Input, period, multiplier);
		}

		public Indicators.mah_Indicators.mahDualCandlePattern mahDualCandlePattern(ISeries<double> input , int period, double multiplier)
		{
			return indicator.mahDualCandlePattern(input, period, multiplier);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.mah_Indicators.mahDualCandlePattern mahDualCandlePattern(int period, double multiplier)
		{
			return indicator.mahDualCandlePattern(Input, period, multiplier);
		}

		public Indicators.mah_Indicators.mahDualCandlePattern mahDualCandlePattern(ISeries<double> input , int period, double multiplier)
		{
			return indicator.mahDualCandlePattern(input, period, multiplier);
		}
	}
}

#endregion
