#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.Indicators;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Strategies in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Strategies
{
	public class GoldenTriangle : Strategy
	{
		private bool		firstEntry;
		private bool		priceSignal;
		private SMA			sma;
		private VolumeGT	vGT;
		private VOL			vol;
		private bool		volumeSignal;
		
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description						= @"Charlotte Hudgin : 'Finding The Golden Triangle' - TASC September 2014";
				Name							= "GoldenTriangle";
				Calculate						= Calculate.OnBarClose;
				EntriesPerDirection				= 1;
				EntryHandling					= EntryHandling.AllEntries;
				IsExitOnSessionCloseStrategy	= true;
				ExitOnSessionCloseSeconds		= 30;
				IsFillLimitOnTouch				= false;
				MaximumBarsLookBack				= MaximumBarsLookBack.TwoHundredFiftySix;
				OrderFillResolution				= OrderFillResolution.Standard;
				Slippage						= 0;
				StartBehavior					= StartBehavior.WaitUntilFlat;
				TimeInForce						= TimeInForce.Gtc;
				TraceOrders						= false;
				RealtimeErrorHandling			= RealtimeErrorHandling.StopCancelClose;
				StopTargetHandling				= StopTargetHandling.PerEntryExecution;
				BarsRequiredToTrade				= 20;

				TraceOrders						= true;

				MinSlope						= 0.25;
				ProfitTarget					= 20;
				priceSignal						= false;
				SmaPeriod						= 50;
				StopLoss						= 10;
			}
			else if (State == State.Configure)
			{
				AddChartIndicator(SMA(SmaPeriod));
				AddChartIndicator(VolumeGT(SmaPeriod));
				
				sma = SMA(SmaPeriod);
				vol = VOL();
				vGT  = VolumeGT(SmaPeriod);
			}
			else if (State == State.DataLoaded)
			{
				ChartIndicators[1].Panel 	= 1;
				firstEntry					= true;
				volumeSignal				= false;

				SetStopLoss(CalculationMode.Ticks, StopLoss);
				SetProfitTarget(CalculationMode.Ticks, ProfitTarget);
			}
		}

		protected override void OnBarUpdate()
		{
			if (CurrentBar < SmaPeriod)
				return;

			if (CrossBelow(Close, sma, 1))
				priceSignal = true;
			
			if (vol[0] > vGT.MovingAverage[0])
			{
				for (int i = 1; i < 5; i++)
				{
					if (vol[0] > vol[i])
					{
						volumeSignal = true;
					}
					else
					{
						volumeSignal = false;
						break;
					}
				}
			}
			
			if (priceSignal 
				&& volumeSignal 
				&& Position.MarketPosition == MarketPosition.Flat
				&& Slope(sma, 1, 0) >= MinSlope)
			{
				if (firstEntry)
				{

					EnterLong("Long Entry");
					firstEntry = false;
				}
				
				else if (!firstEntry && BarsSinceExitExecution() > 4)
				{
					EnterLong("Long Entry 2");
				}
			}
		}

		protected override void OnOrderUpdate(Order order, double limitPrice, double stopPrice, int quantity, int filled, double averageFillPrice, OrderState orderState, DateTime time, ErrorCode error, string nativeError)
		{
			if ((order.Name == "Stop loss" && order.OrderState == OrderState.Filled) || (order.Name == "Profit target" && order.OrderState == OrderState.Filled))
			{
				volumeSignal = false;
				priceSignal = false;
			}			
		}

		#region Properties
		[Range(1, int.MaxValue)]
		[NinjaScriptProperty]
		[Display(Name="SmaPeriod", Description="Numbers of bars used for SMA calculations", Order=1, GroupName="Parameters")]
		public int SmaPeriod
		{ get; set; }

		[Range(1, int.MaxValue)]
		[NinjaScriptProperty]
		[Display(Name="ProfitTarget", Description="Profit Target Value (ticks)", Order=3, GroupName="Parameters")]
		public int ProfitTarget
		{ get; set; }

		[Range(1, int.MaxValue)]
		[NinjaScriptProperty]
		[Display(Name="StopLoss", Description="Stop Loss Value (ticks)", Order=4, GroupName="Parameters")]
		public int StopLoss
		{ get; set; }

		[Range(-99, double.MaxValue)]
		[NinjaScriptProperty]
		[Display(Name="MinSlope", Description="Minimum amount of Slope on MA to trade", Order=2, GroupName="Parameters")]
		public double MinSlope
		{ get; set; }
		#endregion

	}
}
