#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Gui.Tools;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.Indicators;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

//This namespace holds Strategies in this folder and is required. Do not change it. 
namespace NinjaTrader.NinjaScript.Strategies
{
	public class MultiDayBreakouts : Strategy
	{
		private List<int> WinningTrades;
		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description									= @"Multi-Day Breakouts strategy as described in the November 2017 Technical Analysis Stocks and Commodities article: Swing Trading Four-Day Breakouts";
				Name										= "Multi-Day Breakouts";
				Calculate									= Calculate.OnBarClose;
				EntriesPerDirection							= 1;
				EntryHandling								= EntryHandling.AllEntries;
				IsExitOnSessionCloseStrategy				= true;
				ExitOnSessionCloseSeconds					= 30;
				IsFillLimitOnTouch							= false;
				MaximumBarsLookBack							= MaximumBarsLookBack.TwoHundredFiftySix;
				OrderFillResolution							= OrderFillResolution.Standard;
				Slippage									= 0;
				StartBehavior								= StartBehavior.WaitUntilFlat;
				TimeInForce									= TimeInForce.Gtc;
				TraceOrders									= false;
				RealtimeErrorHandling						= RealtimeErrorHandling.StopCancelClose;
				StopTargetHandling							= StopTargetHandling.PerEntryExecution;
				BarsRequiredToTrade							= 20;
				// Disable this property for performance gains in Strategy Analyzer optimizations
				// See the Help Guide for additional information
				IsInstantiatedOnEachOptimizationIteration	= true;
				LookBack									= 4;
				TrailStop									= 2;
				SubmissionOffset							= 0.50;
				TrendsToIncrease							= 20;
				QuantityToIncrement							= 10;
				Quantity									= 10;
				PositionSizing								= true;
			}
			else if (State == State.DataLoaded)
			{
				WinningTrades = new List<int>();
			}
		}

		protected override void OnBarUpdate()
		{
			if (CurrentBar < LookBack)
				return;
			
			int UpCount = 0;
			for (int i = LookBack; i > 0; i--)
				if (Close[i] < Close[i-1])
					UpCount++;
			
			if(UpCount >= LookBack)
			{
				int FirstBoxBar = LookBack - 1;
				double SubmissionPrice = High[0] + SubmissionOffset;
				
				for (int i = 0; i < LookBack; i++)
					BarBrushes[i] = Brushes.DarkGreen;
				
				for (int i = 0; i < CurrentBar; i++)
					if(Close[i+1] > Close[i])
					{
						FirstBoxBar = i-1;
						break;
					}
				
				Draw.Rectangle(this, "Rectangle"+(CurrentBar-FirstBoxBar), true, FirstBoxBar, MIN(Low, FirstBoxBar+1)[0], 0, MAX(High, FirstBoxBar+1)[0], Brushes.DarkGreen, Brushes.LightGreen, 40);
				Draw.Region(this, "Region" + (CurrentBar-FirstBoxBar), FirstBoxBar, 0,Low, High, Brushes.DarkGreen, Brushes.LightGreen, 40);
					
				if (!WinningTrades.Contains((CurrentBar-FirstBoxBar)))
					WinningTrades.Add((CurrentBar-FirstBoxBar));
				
				if(WinningTrades.Count % TrendsToIncrease == 0 && PositionSizing == true)
					Quantity += QuantityToIncrement;
				
				Draw.TextFixed(this, "WinningTradeCounter", "Number of Up Trends: " + WinningTrades.Count, TextPosition.BottomLeft);
				
				Draw.Dot(this, "Entry"+(CurrentBar-FirstBoxBar), true, 0, SubmissionPrice, Brushes.Goldenrod); 
				
				EnterLongStopMarket(0,true,Quantity,SubmissionPrice, "Entry");
				SetTrailStop("Entry", CalculationMode.Ticks, TrailStop/TickSize, false);
			}
				
		}

		#region Properties
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="LookBack", Description="Bars to look lack to detect trend", Order=1, GroupName="Parameters")]
		public int LookBack
		{ get; set; }
		
		[NinjaScriptProperty]
		[Range(0, double.MaxValue)]
		[Display(Name="TrailStop", Description="Currency unit used for Trail Stop", Order=2, GroupName="Parameters")]
		public double TrailStop
		{ get; set; }
		
		[NinjaScriptProperty]
		[Range(0, double.MaxValue)]
		[Display(Name="SubmissionOffset", Description="Offset of Entry Price from trend High", Order=3, GroupName="Parameters")]
		public double SubmissionOffset
		{ get; set; }
		
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="TrendsToIncrease", Description="Number of up trends needed to increment order quantity", Order=4, GroupName="Parameters")]
		public int TrendsToIncrease
		{ get; set; }
		
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="QuantityToIncrement", Description="Quantity to increase order quantity by", Order=5, GroupName="Parameters")]
		public int QuantityToIncrement
		{ get; set; }
		
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="Quantity", Description="Initial Order Quantity", Order=6, GroupName="Parameters")]
		public int Quantity
		{ get; set; }
		
		[NinjaScriptProperty]
		[Range(1, int.MaxValue)]
		[Display(Name="PositionSizing", Description="Increase Order Quantity with each UpTrend", Order=7, GroupName="Parameters")]
		public bool PositionSizing
		{ get; set; }
		#endregion

	}
}
